@@ -1,8 +1,7 @@
 import sqlite3
-import uuid
 from pkg_resources import parse_version
 
-__version__ = "0.1.2"
+__version__ = "0.2.0"
 __author__ = "Mickael Desfrenes"
 __email__ = "desfrenes@gmail.com"
 
@@ -20,13 +19,20 @@ class SQLiteWriter(object):
         self.db = sqlite3.connect(db_path)
         self.db.row_factory = sqlite3.Row
         self.frozen = frozen
-        self.db.cursor().execute("PRAGMA foreign_keys=ON")
+        self.cursor = self.db.cursor()
+        self.cursor.execute("PRAGMA foreign_keys=ON;")
+        self.cursor.execute('PRAGMA encoding = "UTF-8";')
     def __del__(self):
         self.db.close()
 
     def replace(self, bean):
         keys = []
         values = []
+        write_operation = "replace"
+        if "id" not in bean.__dict__:
+            write_operation = "insert"
+            keys.append("id")
+            values.append(None)
         self.__create_table(bean.__class__.__name__)
         columns = self.__get_columns(bean.__class__.__name__)
         for key in bean.__dict__:
@@ -34,16 +40,14 @@ class SQLiteWriter(object):
             if key not in columns:
                 self.__create_column(bean.__class__.__name__, key,
                         type(bean.__dict__[key]))
-            if isinstance(bean.__dict__[key], uuid.UUID):
-                values.append(sqlite3.Binary(bean.__dict__[key].bytes))
-            else:
-                values.append(bean.__dict__[key])
-        cursor = self.db.cursor()
-        sql  = "replace into " + bean.__class__.__name__ + "(" 
+            values.append(bean.__dict__[key])
+        sql  = write_operation + " into " + bean.__class__.__name__ + "("
         sql += ",".join(keys) + ") values (" 
         sql += ",".join(["?" for i in keys])  +  ")"
-        cursor.execute(sql, values)
-        self.db.commit()
+        self.cursor.execute(sql, values)
+        if write_operation == "insert":
+            bean.id = self.cursor.lastrowid
+        return bean.id
 
     def __create_column(self, table, column, sqltype):
         if self.frozen:
@@ -53,34 +57,30 @@ class SQLiteWriter(object):
         else:
             sqltype = "TEXT"
         sql = "alter table " + table + " add " + column + " " + sqltype    
-        self.db.cursor().execute(sql)
-        self.db.commit()
+        self.cursor.execute(sql)
 
     def __get_columns(self, table):
         columns = []
         if self.frozen:
             return columns
-        cursor = self.db.cursor()
-        cursor.execute("PRAGMA table_info(" + table  + ")")
-        for row in cursor:
+        self.cursor.execute("PRAGMA table_info(" + table  + ")")
+        for row in self.cursor:
             columns.append(row["name"])
         return columns
 
     def __create_table(self, table):
         if self.frozen:
             return
-        sql = "create table if not exists " + table + "(uuid primary key)"
-        self.db.cursor().execute(sql)
-        self.db.commit()
+        sql = "create table if not exists " + table + "(id INTEGER PRIMARY KEY AUTOINCREMENT)"
+        self.cursor.execute(sql)
 
     def get_rows(self, table_name, sql = "1", replace = None):
         if replace is None : replace = []
         self.__create_table(table_name)
         sql = "SELECT * FROM " + table_name + " WHERE " + sql
-        cursor = self.db.cursor()
         try:
-            cursor.execute(sql, replace)
-            for row in cursor:
+            self.cursor.execute(sql, replace)
+            for row in self.cursor:
                 yield row
         except sqlite3.OperationalError:
             return
@@ -88,20 +88,18 @@ class SQLiteWriter(object):
     def get_count(self, table_name, sql="1", replace = None):
         if replace is None : replace = []
         self.__create_table(table_name)
-        cursor = self.db.cursor()
         sql = "SELECT count(*) AS cnt FROM " + table_name + " WHERE " + sql
         try:
-            cursor.execute(sql, replace)
+            self.cursor.execute(sql, replace)
         except sqlite3.OperationalError:
             return 0
-        for row in cursor:
+        for row in self.cursor:
             return row["cnt"]
 
     def delete(self, bean):
         self.__create_table(bean.__class__.__name__)
-        sql = "delete from " + bean.__class__.__name__ + " where uuid=?"
-        self.db.cursor().execute(sql,[sqlite3.Binary(bean.uuid.bytes)])
-        self.db.commit()
+        sql = "delete from " + bean.__class__.__name__ + " where id=?"
+        self.cursor.execute(sql,[bean.id])
     
     def link(self, bean_a, bean_b):
         self.replace(bean_a)
@@ -109,66 +107,68 @@ class SQLiteWriter(object):
         table_a = bean_a.__class__.__name__
         table_b = bean_b.__class__.__name__
         assoc_table = self.__create_assoc_table(table_a, table_b)
-        sql = "replace into " + assoc_table + "(" + table_a + "_uuid," + table_b
-        sql += "_uuid) values(?,?)"
-        self.db.cursor().execute(sql, 
-                [buffer(bean_a.uuid.bytes), buffer(bean_b.uuid.bytes)])
-        self.db.commit()
+        sql = "replace into " + assoc_table + "(" + table_a + "_id," + table_b
+        sql += "_id) values(?,?)"
+        self.cursor.execute(sql,
+                [bean_a.id, bean_b.id])
     
     def unlink(self, bean_a, bean_b):
         table_a = bean_a.__class__.__name__
         table_b = bean_b.__class__.__name__
         assoc_table = self.__create_assoc_table(table_a, table_b)
         sql = "delete from " + assoc_table + " where " + table_a
-        sql += "_uuid=? and " + table_b + "_uuid=?"
-        self.db.cursor().execute(sql,
-                [buffer(bean_a.uuid.bytes), buffer(bean_b.uuid.bytes)]) 
-        self.db.commit()
+        sql += "_id=? and " + table_b + "_id=?"
+        self.cursor.execute(sql,
+                [bean_a.id, bean_b.id])
+        #self.db.commit()
     
     def get_linked_rows(self, bean, table_name):
         bean_table = bean.__class__.__name__
         assoc_table = self.__create_assoc_table(bean_table, table_name)
-        cursor = self.db.cursor()
         sql = "select t.* from " + table_name + " t inner join " + assoc_table 
-        sql += " a on a." + table_name + "_uuid = t.uuid where a." 
-        sql += bean_table + "_uuid=?"
-        cursor.execute(sql,[buffer(bean.uuid.bytes)])
-        for row in cursor:
+        sql += " a on a." + table_name + "_id = t.id where a."
+        sql += bean_table + "_id=?"
+        self.cursor.execute(sql,[bean.id])
+        for row in self.cursor:
             yield row
 
     def __create_assoc_table(self, table_a, table_b):
         assoc_table = "_".join(sorted([table_a, table_b]))
         if not self.frozen:
             sql = "create table if not exists " + assoc_table + "("
-            sql+= table_a + "_uuid NOT NULL REFERENCES " + table_a + "(uuid) ON DELETE cascade,"
-            sql+= table_b + "_uuid NOT NULL REFERENCES " + table_b + "(uuid) ON DELETE cascade,"
-            sql+= " PRIMARY KEY (" + table_a + "_uuid," + table_b + "_uuid));"
-            self.db.cursor().execute(sql)
+            sql+= table_a + "_id NOT NULL REFERENCES " + table_a + "(id) ON DELETE cascade,"
+            sql+= table_b + "_id NOT NULL REFERENCES " + table_b + "(id) ON DELETE cascade,"
+            sql+= " PRIMARY KEY (" + table_a + "_id," + table_b + "_id));"
+            self.cursor.execute(sql)
             # no real support for foreign keys until sqlite3 v3.6.19
             # so here's the hack
             if cmp(parse_version(sqlite3.sqlite_version),parse_version("3.6.19")) < 0:
                 sql = "create trigger if not exists fk_" + table_a + "_" + assoc_table
                 sql+= " before delete on " + table_a
-                sql+= " for each row begin delete from " + assoc_table + " where " + table_a + "_uuid = OLD.uuid;end;"
-                self.db.cursor().execute(sql)
+                sql+= " for each row begin delete from " + assoc_table + " where " + table_a + "_id = OLD.id;end;"
+                self.cursor.execute(sql)
                 sql = "create trigger if not exists fk_" + table_b + "_" + assoc_table
                 sql+= " before delete on " + table_b
-                sql+= " for each row begin delete from " + assoc_table + " where " + table_b + "_uuid = OLD.uuid;end;"
-                self.db.cursor().execute(sql)
-        self.db.commit()        
+                sql+= " for each row begin delete from " + assoc_table + " where " + table_b + "_id = OLD.id;end;"
+                self.cursor.execute(sql)
         return assoc_table
 
     def delete_all(self, table_name, sql = "1", replace = None):
         if replace is None : replace = []
         self.__create_table(table_name)
-        cursor = self.db.cursor()
         sql = "DELETE FROM " + table_name + " WHERE " + sql
         try:
-            cursor.execute(sql, replace)
+            self.cursor.execute(sql, replace)
             return True
         except sqlite3.OperationalError:
             return False
 
+    def commit(self):
+        self.db.commit()
+
+    def begin(self):
+        self.cursor.execute('BEGIN;');
+
 class Store(object):
     """
     A SQL writer should be passed to the constructor:
@@ -180,14 +180,13 @@ class Store(object):
     
     def new(self, table_name):
         new_object = type(table_name,(object,),{})()
-        new_object.uuid = uuid.uuid4()
         return new_object
 
     def save(self, bean):
         self.writer.replace(bean)
     
-    def load(self, table_name, uuid):
-        for row in self.writer.get_rows(table_name, "uuid=?", [buffer(uuid.bytes)]):
+    def load(self, table_name, id):
+        for row in self.writer.get_rows(table_name, "id=?", [id]):
             return self.row_to_object(table_name, row)
 
     def count(self, table_name, sql = "1", replace=None):
@@ -222,8 +221,11 @@ class Store(object):
     def row_to_object(self, table_name, row):
         new_object = type(table_name,(object,),{})()
         for key in row.keys():
-            if key == "uuid":
-                new_object.uuid = uuid.UUID(bytes=row[key])
-            else:
-                new_object.__dict__[key] = row[key]
+            new_object.__dict__[key] = row[key]
         return new_object
+
+    def begin(self):
+        self.writer.begin()
+
+    def commit(self):
+        self.writer.commit()
